'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "gamelib"
package.
'''

import data
import util
import math
from random import choice
import numpy
import pyglet.window.key
import pyglet
import pyglet.clock
import pyglet.sprite
import pyglet.resource
from pyglet.window import *
from pyglet.gl import *
from pyglet.media import *

WINDOWWIDTH = 800
WINDOWHEIGHT = 600
GAME_TITLE = "Wibbly-Wobbly Boxing Penguin of DOOM"
VERSION = 0.1

window = pyglet.window.Window(width = WINDOWWIDTH, height = WINDOWHEIGHT)
window.set_caption(GAME_TITLE + " v." + str(VERSION))
window.set_mouse_visible(False)

glEnable(GL_BLEND)
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

current_screen = None


class FieldSection(pyglet.sprite.Sprite):
    def __init__(self):
        global field_section_perfect
        self.image = field_section_perfect
        self.hp = 100
        

    def update(self, damage):
        global field_section_critical, field_section_damaged, field_section_gone
        self.hp -= damage
        if self.hp < 1:
            self.image = field_section_gone
        elif self.hp < 25:
            self.image = field_section_critical
        elif self.hp < 50:
            self.image = field_section_damaged

field_section_gone = util.load_centered_image(data.filepath("pit.png"))
field_section_critical = util.load_centered_image(data.filepath("ice3.png"))
field_section_damaged = util.load_centered_image(data.filepath("ice2.png"))
field_section_perfect = util.load_centered_image(data.filepath("ice1.png"))
class Field:
    def __init__(self):

        self.x = WINDOWWIDTH /2
        self.delta_x = WINDOWWIDTH / 8
        self.y = WINDOWHEIGHT / 2
        self.delta_y = WINDOWHEIGHT / 6
        self.grid = numpy.zeros((8,6))
        
    def update(self, subject):
        x_loc = subject.x / 100
        y_loc = subject.y / 100
        if x_loc < 8 and y_loc < 6:
            self.grid[x_loc][y_loc] += 1
            if self.grid[x_loc][y_loc] > 300:
                subject.die()

    def draw(self):
        global field_section_perfect, field_section_damaged, field_section_critical, field_section_gone
        for w in range(0, 8):
            for h in range(0,6):
                if self.grid[w][h] > 300:
                    field_section_gone.blit(w * self.delta_x + self.delta_x / 2, h * self.delta_y + self.delta_y / 2)
                elif self.grid[w][h] > 200:
                    field_section_critical.blit(w * self.delta_x + self.delta_x / 2, h * self.delta_y + self.delta_y / 2)
                elif self.grid[w][h] > 100:
                    field_section_damaged.blit(w * self.delta_x + self.delta_x / 2, h * self.delta_y + self.delta_y / 2)
                else:
                    field_section_perfect.blit(w * self.delta_x + self.delta_x / 2, h * self.delta_y + self.delta_y / 2)


#                self.label.x = w * self.delta_x + self.delta_x / 2
#                self.label.y = h * self.delta_y + self.delta_y / 2
#                self.label.draw()

#penguin sounds
penguin_sound_jump = []
penguin_sound_jump.append(StaticSource(load(data.filepath('jump1.mp3'))))
penguin_sound_jump.append(StaticSource(load(data.filepath('jump2.mp3'))))
penguin_sound_jump.append(StaticSource(load(data.filepath('jump3.mp3'))))
penguin_sound_punch = StaticSource(load(data.filepath('punch.mp3')))
penguin_sound_pain = []
penguin_sound_pain.append(StaticSource(load(data.filepath('pain1.mp3'))))
penguin_sound_pain.append(StaticSource(load(data.filepath('pain2.mp3'))))
penguin_sound_pain.append(StaticSource(load(data.filepath('pain3.mp3'))))
penguin_sound_pain.append(StaticSource(load(data.filepath('pain4.mp3'))))
penguin_sound_pain.append(StaticSource(load(data.filepath('pain5.mp3'))))
penguin_sound_die = StaticSource(load(data.filepath('death.mp3')))
class Penguin(pyglet.sprite.Sprite):
    RIGHT = 0
    FOWARD = 1
    LEFT = 2
    BACKWARD = 3
    RIGHT_P = 4
    FOWARD_P = 5
    LEFT_P = 6
    BACKWARD_p = 7
    HIT = 8

    def __init__(self, images):
        global penguin_down
        self.images = images
        pyglet.sprite.Sprite.__init__(self, self.images[self.FOWARD])
        self.x_momentum = 0
        self.y_momentum = 0
        self.facing = self.FOWARD
        self.alive = 1
        self.slapping = 0
        self.hit = 0
        self.label = pyglet.text.Label("slap", font_size=40.0, anchor_x="center", anchor_y="center")

    def move(self, direction):
        max_speed = 5
        #up
        if direction is 0:
            self.facing = self.BACKWARD
            if self.y_momentum < max_speed:
                self.y_momentum += 1
        #right
        elif direction is 1:
            self.facing = self.RIGHT
            if self.x_momentum < max_speed:
                self.x_momentum += 1
        #down
        elif direction is 2:
            self.facing = self.FOWARD
            if self.y_momentum > -1 * max_speed:
                self.y_momentum -= 1
        #left
        elif direction is 3:
            self.facing = self.LEFT
            if self.x_momentum > -1 * max_speed:
                self.x_momentum -= 1

    def slap(self):
        global penguin_sound_punch
        if self.slapping == 0:
            self.slapping = 15
            penguin_sound_punch.play()
            

    def update(self, target):
        global penguin_normal, penguin_sound_pain
        if self.alive is 1:
            #handle borders
            if self.x < 0 and self.x_momentum < 0:
                self.mirror_velocity(1, 0)
            elif self.x > 800 and self.x_momentum > 0:
                self.mirror_velocity(1, 0)
            if self.y < 0 and self.y_momentum < 0:
                self.mirror_velocity(0, 1)
            elif self.y > 600 and self.y_momentum > 0:
                self.mirror_velocity(0, 1)

            #move the penguin
            self.x += self.x_momentum
            self.y += self.y_momentum

            #handle slapping
            if self.slapping > 0:
                self.slapping -= 1
                #assuming targets are all squares
                if self.slapping > 5:
                    range = 100
                    if self.slapping is 13:
                        if self.facing is self.FOWARD:
                            print target.x, self.x
                            if math.fabs(target.x - self.x) < target.width and self.y - target.y < range and math.fabs(self.y - target.y) > self.height:
                                target.x_momentum = 0
                                target.y_momentum = -10
                                choice(penguin_sound_pain).play()
                                target.hit = 10
                        elif self.facing is self.BACKWARD:
                            if math.fabs(target.x - self.x) < target.width and self.y - target.y > -1 * range and math.fabs(self.y - target.y)  > self.height:
                                target.x_momentum = 0
                                target.y_momentum = 10
                                choice(penguin_sound_pain).play()
                                target.hit = 10
                        elif self.facing is self.LEFT:
                            if math.fabs(self.x - target.x) < range and self.x - target.x > -1 * self.width and math.fabs(target.y - self.y) < target.height:
                                target.x_momentum = -10
                                target.y_momentum = 0
                                choice(penguin_sound_pain).play()
                                target.hit = 10        
                        elif self.facing is self.RIGHT:
                            if math.fabs(self.x - target.x) < range and self.x - target.x < self.width and math.fabs(target.y - self.y) < target.height:
                                target.x_momentum = 10
                                target.y_momentum = 0
                                choice(penguin_sound_pain).play()
                                target.hit = 10    
        else: #penguin is dead
            self.rotation += 10
            self.scale -= 0.01
            self.alive -= 1

    def mirror_velocity(self, horz=0, vert=0):
        global penguin_sound_jump
        choice(penguin_sound_jump).play()
        if horz is 1:
            self.x_momentum = self.x_momentum * -1
        if vert is 1:
            self.y_momentum = self.y_momentum * -1

    def draw(self):
        if self.image is not self.images[self.facing]:
            self.image = self.images[self.facing]
        if self.slapping > 0:
            self.image = self.images[self.facing + 4]
        if self.hit > 0:
            self.image = self.images[self.HIT]
            self.hit -= 1
            
        pyglet.sprite.Sprite.draw(self)

    def die(self):
        global penguin_sound_die
        if self.alive > 0:
            self.alive = -1
            penguin_sound_die.play()
            

class Screen:
    sprite_batch = None

    def __init__(self):
        pass

    def draw(self):
        pass

    def on_mouse(self, x, y, button, modifiers):
        pass

    def on_key(self, symbol, modifier):
        pass

    def on_text_motion(self, motion):
        pass

    def on_text(self, text):
        pass


class Menu(Screen):
    def __init__(self):
        bg_img = pyglet.image.load(data.filepath("main.png"))
        self.background = pyglet.sprite.Sprite(bg_img)

    def draw(self):
        self.background.draw()

    def on_key(self, symbol, modifiers):
        global current_screen
        if symbol is pyglet.window.key.B:
            #start the game
            current_screen = Game()
        elif symbol is pyglet.window.key.C:
            current_screen = Credits()
        elif symbol is pyglet.window.key.H:
            current_screen = Help()

class Help(Screen):
    def __init__(self):
        bg_img = pyglet.image.load(data.filepath("helpz.png"))
        self.background = pyglet.sprite.Sprite(bg_img)

    def draw(self):
        self.background.draw()

    def on_key(self, symbol, modifiers):
        global current_screen
        if symbol is pyglet.window.key.M:
            #start the game
            current_screen = Menu()

class Credits(Screen):
    def __init__(self):
        bg_img = pyglet.image.load(data.filepath("credits.png"))
        self.background = pyglet.sprite.Sprite(bg_img)

    def draw(self):
        self.background.draw()

    def on_key(self, symbol, modifiers):
        global current_screen
        if symbol is pyglet.window.key.M:
            #start the game
            current_screen = Menu()

game_music = StaticSource(load(data.filepath('mktheme.mp3')))
class Game(Screen):
    #game state
    READY = 0
    PLAYING = 1
    FINISHED = 2
    game_state = READY
    
    def __init__(self):
        global game_music
        self.music = game_music.play()
        #Stuff for ready state        
        self.ready_label = pyglet.text.Label("Prepare for death!\nPress 'Space' to start.", font_size=24.0, anchor_x="center", anchor_y="center")
        self.ready_label.x = WINDOWWIDTH/2
        self.ready_label.y = WINDOWHEIGHT/2

        #Playing state
        self.last_collision = 0
        self.field = Field()

        penguin_images = []
        penguin_images.append(util.load_centered_image(data.filepath("right.png")))
        penguin_images.append(util.load_centered_image(data.filepath("forward.png")))
        penguin_images.append(util.load_centered_image(data.filepath("left.png")))
        penguin_images.append(util.load_centered_image(data.filepath("back.png")))
        penguin_images.append(util.load_centered_image(data.filepath("rightpunch.png")))
        penguin_images.append(util.load_centered_image(data.filepath("forwardpunch.png")))
        penguin_images.append(util.load_centered_image(data.filepath("leftpunch.png")))
        penguin_images.append(util.load_centered_image(data.filepath("backpunch.png")))
        penguin_images.append(util.load_centered_image(data.filepath("hit.png")))
        self.wibbly = Penguin(penguin_images)
        self.wibbly.x = 50
        self.wibbly.y = 50
        self.wibbly.scale = 2.0

        penguin_images = []
        penguin_images.append(util.load_centered_image(data.filepath("right2.png")))
        penguin_images.append(util.load_centered_image(data.filepath("forward2.png")))
        penguin_images.append(util.load_centered_image(data.filepath("left2.png")))
        penguin_images.append(util.load_centered_image(data.filepath("back2.png")))
        penguin_images.append(util.load_centered_image(data.filepath("rightpunch2.png")))
        penguin_images.append(util.load_centered_image(data.filepath("forwardpunch2.png")))
        penguin_images.append(util.load_centered_image(data.filepath("leftpunch2.png")))
        penguin_images.append(util.load_centered_image(data.filepath("backpunch2.png")))
        penguin_images.append(util.load_centered_image(data.filepath("hit2.png")))
        self.wobbly = Penguin(penguin_images)
        self.wobbly.x = 750
        self.wobbly.y = 550
        self.wobbly.scale = 2.0

        #Finished state
        self.finished_label = pyglet.text.Label("Congradulations!.", font_size=24.0, anchor_x="center", anchor_y="center")
        self.finished_label.x = WINDOWWIDTH/2
        self.finished_label.y = WINDOWHEIGHT/2

    def on_key(self, symbol, modifiers):
        global current_screen
        if  self.game_state is self.READY:
            if symbol is pyglet.window.key.SPACE:
                self.game_state = self.PLAYING
        elif self.game_state is self.PLAYING:
            #wobbly's movements
            if symbol is pyglet.window.key.UP:
                self.wobbly.move(0)
            elif symbol is pyglet.window.key.DOWN:
                self.wobbly.move(2)
            elif symbol is pyglet.window.key.LEFT:
                self.wobbly.move(3)
            elif symbol is pyglet.window.key.RIGHT:
                self.wobbly.move(1)
            elif symbol is pyglet.window.key.RCTRL:
                self.wobbly.slap()
            #wibbly's movements
            if symbol is pyglet.window.key.W:
                self.wibbly.move(0)
            elif symbol is pyglet.window.key.S:
                self.wibbly.move(2)
            elif symbol is pyglet.window.key.A:
                self.wibbly.move(3)
            elif symbol is pyglet.window.key.D:
                self.wibbly.move(1)
            elif symbol is pyglet.window.key.LCTRL:
                self.wibbly.slap()
        elif self.game_state is self.FINISHED:
            if symbol is pyglet.window.key.SPACE:
                self.music.stop()
                current_screen = Game()

    def on_text(self, text):
        if text == 'w':
            self.wibbly.move(0)
        elif text == 's':
            self.wibbly.move(2)
        elif text == 'a':
            self.wibbly.move(3)
        elif text == 'd':
            self.wibbly.move(1)

    def on_text_motion(self, motion):
        if self.game_state is self.PLAYING:
            if motion is pyglet.window.key.MOTION_UP:
                self.wobbly.move(0)
            elif motion is pyglet.window.key.MOTION_DOWN:
                self.wobbly.move(2)
            elif motion is pyglet.window.key.LEFT:
                self.wobbly.move(3)
            elif motion is pyglet.window.key.RIGHT:
                self.wobbly.move(1)
                


    def game_logic(self):
        #leave dead penguin animating for 100 frames
        if self.wibbly.alive < -100:
            self.wibbly = None
        if self.wobbly.alive < -100:
            self.wobbly = None
        #check for end game condition
        if self.wibbly is None or self.wobbly is None:
            self.game_state = self.FINISHED
            return

#        if self.last_collision > 0:
#            self.last_collision -= 1
        if util.collide(self.wibbly, self.wobbly) and self.last_collision is 0:
            tmp_x = self.wibbly.x_momentum
            tmp_y = self.wibbly.y_momentum
            self.wibbly.x_momentum = self.wobbly.x_momentum
            self.wibbly.y_momentum = self.wobbly.y_momentum
            self.wobbly.x_momentum = tmp_x
            self.wobbly.y_momentum = tmp_y
#            self.last_collision = 30

        self.wibbly.update(self.wobbly)
        self.wobbly.update(self.wibbly)
        self.field.update(self.wobbly)
        self.field.update(self.wibbly)

        

    def draw(self):
        #always show the penguins and field
        self.field.draw()
        if self.wibbly is not None:
            self.wibbly.draw()
        if self.wobbly is not None:
            self.wobbly.draw()
        
        if self.game_state is self.READY:
            self.ready_label.draw()
        elif self.game_state is self.PLAYING:
            self.game_logic()
        elif self.game_state is self.FINISHED:
            if self.wibbly is None:
                self.finished_label.text = "Congrads Wobbly! Space to play again!"
            else:
                self.finished_label.text = "Congrads Wibbly! Space to play again!"
            self.finished_label.draw()

@window.event
def on_mouse_press(x, y, button, modifiers):
    #Not using mouse on anything
    pass
    
@window.event
def on_key_press(symbol, modifiers):
    global current_screen
    current_screen.on_key(symbol, modifiers)
    
@window.event
def on_text(text):
    global current_screen
    current_screen.on_text(text)
    
@window.event
def on_text_motion(motion):
    global current_screen
    current_screen.on_text_motion(motion)

def update(dt):
    window.clear()

    current_screen.draw()
    fps_display.draw()

    #window.flip()

#window.push_handlers(pyglet.window.event.WindowEventLogger())
fps_display = pyglet.clock.ClockDisplay()
pyglet.clock.schedule_interval_soft(update, 1/120.)

def main():
    global current_screen
    current_screen = Menu()
    pyglet.app.run()
